#include "st_sys.h"
#include "st_flash.h"
#include "hardware_delay.h"

typedef struct
{
    uint32_t addr;
    uint32_t size;
}

st_flash_page_info;



/**
 * FLASHҳֿ
 *
 * @author lxz (121319 18:16:08)
 */
static const st_flash_page_info page_infos[] = {
    { ST_FLASH_SECTOR_0, 24 * 1024},
    { ST_FLASH_SECTOR_2, 16 * 1024},
    { ST_FLASH_SECTOR_3, 4 * 1024},
    { ST_FLASH_SECTOR_5, 212 * 1024 }
};

/**
 * flash 
 *
 * @author lxz (2019/5/30/周四)
 *
 * @param void
 */
void st_flash_unlock(void) {

    if (FLASH->CR & (1 << 7)) {
        FLASH->KEYR = FLASH_KEY1;
        FLASH->KEYR = FLASH_KEY2;
    }
    //־λ
    FLASH->SR = FLASH_SR_PGERR |FLASH_SR_WRPRTERR;
}

/**
 * flash 
 *
 * @author lxz (2019/5/30/周四)
 *
 * @param void
 */
void st_flash_lock(void) {
    FLASH->CR |= ((uint32_t)1) << 7;   //Bank1,上锁
}

/**
 * ȡflash״̬
 *
 * @author LXZ (121819)
 *
 * @return int
 */
int st_flash_error_no(void) {
    int res = 0;
    res = FLASH->SR;

    if (res & (1 << 0)) return 1;        //æ
    else if (res & (1 << 2)) return 2;    //ҳ
    else if (res & (1 << 4)) return 3;    //д
    return 0;                       //ûκ״̬/
}
int flash_last_wait_time = 0;
/**
 * ȴ
 *
 * @author lxz (2019/5/30/周四)
 *
 * @param bankx
 * @param time ȴʱ
 */
int st_flash_wait_done(int time) {
    int res = 0;

    do {
        res = st_flash_error_no();
        if (res != 1) break; //æ,ȴ,ֱ˳.
        hw_delay_us(1);
        time--;
    }while (time);
    if (time == 0) res = 0xff; //TIMEOUT
    flash_last_wait_time = time;
    return res;
}

/**
 *
 *
 * @author lxz (2019/5/30/周四)
 *
 * @param sector_no
 *
 * @return int
 */
int st_flash_erase_secotr(uint8_t sector_no) {
    int res = 0;
	  int number = 0;
      int blockNumber = page_infos[sector_no].size;
	  uint32_t addr = page_infos[sector_no].addr;
          int page_size = 1024;
    //FLASH->ACR &= ~(1 << 4);
    res = st_flash_wait_done(20000); //等待上次操作结束,最?s

    if (__ST_FLASH_SIZE >= 0x100) {
        page_size = 2048;

    }
    blockNumber /= page_size;

    while (res == 0 && number < blockNumber) {

        //FLASH->CR&=~(3<<8); //PSIZEԭ
        //FLASH->CR|=2<<8;    //Ϊ32bit,ȷVCC=2.7~3.6V֮!!
        //FLASH->CR&=~(0X1F<<3);//ԭ
        //FLASH->CR|=sector_no<<3;//Ҫ
        FLASH->CR |= 1 << 1;    //
        FLASH->AR = addr;
        FLASH->CR |= 1 << 6;   //ʼ
        res = st_flash_wait_done(20000); //ȴ,2s
        if (res != 1) {          //æ
            FLASH->CR &= ~(1 << 1); //־.
        }
        number++;
        addr += page_size;
    }
    //FLASH->ACR |= (1 << 4);
    return res;
}

/**
 * ַָдһַ֣뱣ֶ֤
 *
 * @author lxz (2019/5/30/周四)
 *
 * @param faddr
 * @param pdata
 *
 * @return int
 */
int st_flash_write_flash_word(uint32_t faddr, uint32_t pdata) {
    u8 res;
    res = st_flash_wait_done(0XFF);
    if (res == 0) { //OK
        FLASH->CR |= 1 << 0;
        *(vu16 *)faddr = pdata;
        res = st_flash_wait_done(0XFF);
        if (res == 0) {
            FLASH->CR &= ~(1 << 0);
        }

    }
    res = st_flash_wait_done(0XFF);
    faddr += 2;
    pdata >>= 16;
    if (res == 0) {
        FLASH->CR |= 1 << 0;
        *(vu16 *)faddr = pdata;
        res = st_flash_wait_done(0XFF);
        if (res == 0) {
            FLASH->CR &= ~(1 << 0);
        }
    }
    return res;
}

int st_flash_write_half_word(uint32_t faddr, uint16_t pdata) {
    u8 res;
    res = st_flash_wait_done(0XFF);
    if (res == 0) { //OK
        FLASH->CR |= 1 << 0;
        *(vu16 *)faddr = pdata;
        res = st_flash_wait_done(0XFF);
        if (res == 0) {
            FLASH->CR &= ~(1 << 0);
        }

    }
    return res;
}

/**
 * ȡַָһ32λ
 *
 * @author LXZ (121819)
 *
 * @param faddr
 *
 * @return uint32_t
 */
uint32_t st_flash_read_word(uint32_t faddr) {
    return *(vu32 *)faddr;
}

/**
 * 通过地址获取所在的扇区?
 *
 * @author lxz (2019/5/30/周四)
 *
 * @param addr
 *
 * @return uint8_t
 */
uint8_t st_flash_addr_to_sector_no(uint32_t addr) {

    int  i = 0;

    while ((page_infos[i].addr + page_infos[i].size) <= addr &&
           i < (sizeof(page_infos) / sizeof(st_flash_page_info))) {    //СǷѾ˵ַ
        i++;
    }

    return i;
}

/**
 * ַָд32λ
 *
 * @author LXZ (121819)
 *
 * @param addr
 * @param buffer
 * @param length
 *
 * @return int
 */
int st_flash_write(uint32_t addr, uint32_t *buffer, uint32_t length)   {
    int status = 0;
    uint32_t addrx = 0;
    uint32_t endaddr = 0;
    if (addr < STM32_FLASH_BASE || addr % 4) return 1;  //Ƿַ
    st_flash_unlock();                                    //
    //FLASH->ACR &= ~(1 << 10);           //FLASHڼ,ֹݻ!!!ϲŷ!
    addrx = addr;                //дʼַ
    endaddr = addr + length * 4; //дĽַ
    if (addrx < 0X1FFF0000) {            //ֻ洢,Ҫִв!!
        while (addrx < endaddr) {        //ɨһϰ.(ԷFFFFFFFFĵط,Ȳ)
            if (st_flash_read_word(addrx) != 0XFFFFFFFF) { //з0XFFFFFFFFĵط,Ҫ
                status = st_flash_erase_secotr(st_flash_addr_to_sector_no(addrx));
                if (status) break;    //
            } else addrx += 4;
        }
    }
    if (status == 0) {
        uint16_t * dat = (uint16_t *)buffer;
        while (addr < endaddr) {
        if (st_flash_write_half_word(addr, *dat)) { //д
                break;  //д쳣
         }
            //sw_timer_now(&timer);
            addr += 2;
            dat++;
        }
            /*
        while (addr < endaddr) { //д
             //sw_timer_now(&timer2);
            if (st_flash_write_flash_word(addr, *buffer)) { //д
                break;  //д쳣
            }
            //sw_timer_now(&timer);
            addr += 4;
            buffer++;
        }
*/
    } else {
        while (1);
    }
    //FLASH->ACR |= 1 << 10;      //FLASH,fetch
    st_flash_lock(); //
    return 0;
}

/**
 * ȡָȵ32λ
 *
 * @author LXZ (121819)
 *
 * @param addr
 * @param buffer
 * @param length
 */
void st_flash_read(uint32_t addr, uint32_t *buffer, uint32_t length)     {
    int i;
    for (i = 0; i < length; i++) {
        buffer[i] = st_flash_read_word(addr);
        addr += 4;
    }
}


